﻿using System;
using System.Collections.Generic;
using System.IO;
using Microsoft.Office.Server.Search.ContentProcessingEnrichment;
using Microsoft.Office.Server.Search.ContentProcessingEnrichment.PropertyTypes;

namespace ContentProcessingEnrichmentService
{
    public class CPE : IContentProcessingEnrichmentService
    {
        //The managed property we are looking for
        private const string companyManagedProperty = "FictitiousCompany";

        //The new managed property to add
        private Property<string> tickerSymbol = new Property<string> { Name = "FictitiousTickerSymbol" };


        //A temporary holder for the item being processed
        private readonly ProcessedItem processedItemHolder = new ProcessedItem { ItemProperties = new List<AbstractProperty>() };

        //The dictionary of ticker symbols
        private readonly Dictionary<string, string> tickerSymbolsDictionary = new Dictionary<string, string> {
            {"A. Datum Corporation","MFNLADC"},
            {"Adventure Works", "MFNLAW"},
            {"Alpine Ski House","MFNLASH"},
            {"Blue Yonder Airlines","MFNLBYA"},
            {"City Power & Light","MFNLCPL"},
            {"Coho Vineyard","MFNLCV"},
            {"Coho Winery","MFNLCW"},
            {"Coho Vineyard & Winery","MFNLCVW"},
            {"Contoso, Ltd","MFNLCL"},
            {"Contoso Pharmaceuticals","MFNLCP"},
            {"Consolidated Messenger","MFNLCM"},
            {"Fabrikam, Inc.","MFNLFI"},
            {"Fourth Coffee","MFNLFC"},
            {"Graphic Design Institute","MFNLGDI"},
            {"Humongous Insurance","MFNLHI"},
            {"Litware, Inc.","MFNLLI"},
            {"Lucerne Publishing","MFNLLP"},
            {"Margie's Travel","MFNLMT"},
            {"Northwind Traders","MFNLNT"},
            {"Proseware, Inc.","MFNLPI"},
            {"School of Fine Art","MFNLSFA"},
            {"Southridge Video","MFNLSV"},
            {"Tailspin Toys","MFNLTT"},
            {"Trey Research","MFNLTR"},
            {"The Phone Company","MFNLTPC"},
            {"Wide World Importers","MFNLWWI"},
            {"Wingtip Toys","MFNLWT"},
            {"Woodgrove Bank","MFNLWB"}
        };

        //Error Codes
        private const int UnexpectedType = 1;
        private const int UnexpectedError = 2;

        //Log names
        private const string sourceName = "Content Processing Enrichment Service";
        private const string logName = "Application";

        //Called for each matching item in the pieline
        public ProcessedItem ProcessItem(Item item)
        {
            if (!System.Diagnostics.EventLog.SourceExists(sourceName))
                System.Diagnostics.EventLog.CreateEventSource(sourceName, logName);

            try
            {
                
                foreach (AbstractProperty property in item.ItemProperties)
                {
                    //Determine if this managed property is the company property
                    if (property.Name.Equals(companyManagedProperty, StringComparison.InvariantCultureIgnoreCase))
                    {
                        //Get the managed property
                        Property<string> companyProperty = property as Property<string>;

                        if (companyProperty == null)
                        {
                            //Unexpected Type
                            processedItemHolder.ErrorCode = UnexpectedType;
                            System.Diagnostics.EventLog.WriteEntry(sourceName, "Unexpected type. " + property.Name, System.Diagnostics.EventLogEntryType.Error);
                        }
                        else
                            if (tickerSymbolsDictionary.ContainsKey(property.ObjectValue.ToString()))
                            {
                                //Add the ticker symbol managed property
                                tickerSymbol.Value = tickerSymbolsDictionary[property.ObjectValue.ToString()];
                                processedItemHolder.ItemProperties.Add(tickerSymbol);
                            }
                            else
                            {
                                System.Diagnostics.EventLog.WriteEntry(sourceName, "Could not find " + property.ObjectValue.ToString() + " in the ticker symbol dictionary.", System.Diagnostics.EventLogEntryType.Error);
                            }

                    }
                }
            }
            catch (Exception x)
            {
                //Notify pipeline of error
                processedItemHolder.ErrorCode = UnexpectedError;
                System.Diagnostics.EventLog.WriteEntry(sourceName, "Unexpected error. " + x.Message + " NOTE: The managed property 'TickerSymbol' must already be defined in the Search Service Application for Content Enrichment to work.", System.Diagnostics.EventLogEntryType.Error);
            }

            //Return the processed item to the pipeline
            return processedItemHolder;
        }
    }
}
